#        OLMenu - A menu-driven offline reader system for UQWK
#
#                     Version 1.0.10       03/11/95
#		    Modified for Bourne shell 03/22/95
#
#  Copyright (C) 1995  by  Ken  Gresham <kgresham@america.net>
#  Permission  to  use,  copy, modify  and  distribute  this software
#  and its documentation for any purpose  and  without  fee is hereby
#  granted,  provided  that  the above copyright notice appear in all
#  copies  and  that  both  that copyright notice and this permission
#  notice  appear  in  supporting  documentation.  This  software  is
#  provided "as is" without expressed or implied warranty.
#
#  OLMenu  is  a  menu-driven  offline  reader  system  that operates
#  with  the UQWK message handling software on a Unix host to perform
#  the following functions:
#  
#  1) Configure the offline reader system during the session (configurations
#     are saved upon termination).
#  2) Upload and unzip your REPLIES Packet
#  3) Process your REPLIES packet (sends your email responses and posts
#     to the newsgroups)
#  4) Pack up your new email and the new articles from the newsgroups
#  5) Zip up the packets to make the downloading quicker and easier
#  6) Move the AREA, *.MSG and zipped files after they have been
#     downloaded to the $TrashDir subdirectory of $HOME. (This
#     subdirectory is maintained in case you want to retransfer your 
#     download packet or examine it's component files.) Files in the 
#     $TrashDir subdirectory may be deleted at the beginning of the
#     program if desired.
#
#  File transfer protocols include Xmodem, Ymodem and Zmodem with a shell
#  account, and ftp with or without a server during a telnet session to a
#  shell account.
#
#  Refer to olmenu.howto for detailed instructions.
#
#  Special thanks to Cindy McKinnon who originated the offline reader
#  project and the staff at Access America/Connect Atlanta (tm) who
#  made possible this work.
#
#  Thanks to Dirk A. Loedding (judge@america.net) and others for many
#  valuable criticisms and suggestions.
#
#  Suggestions for future versions and bug reports will be appreciated,
#  and should be e-mailed to kgresham@america.net.

# Assign default constants and variables (some may be configured in OLMenu):

MainHeader="\t\tOLMenu - Offline Reader System"
Dashline=' 
----------------Enter h on any menu to view help 
pages------------------------'
# Uploaded reply packet
ReplyFile="reply.zip"
# Downloaded message packet
NewFile="new.zip"
# Old files moved to this subdirectory
TrashDir="OLTRASH"
# Change path to OLMenu configuration and help files as required.
# Uncomment next line for path in system installation
OLMenuPath='/usr/lib/offline'
# Uncomment next line for path in individual installation
# OLMenuPath=$HOME

DoUpload=NO                     
DoDownload=YES
DefaultProtocol=Zmodem
DoMail=YES
DoNews=NO
ShowMail=YES
ShowNews=NO
Readonly=YES
UQ_MAX_BLKS=4000

# Begin function declarations

ReadEnv()
{
# Get the default configuration from .olmenusysrc (system config)
# and .olmenurc (local config saved after each OLMenu session)
#
# Get the system configuration
. ${OLMenuPath}/.olmenusysrc
#
# Get the saved configuration from the last session
if [ -f $HOME/.olmenurc ]
        then . $HOME/.olmenurc
fi
if [ $DoDownload = NO ]
	then ShowMail="---"
	     ShowNews="---"
	else ShowMail=$DoMail
	     ShowNews=$DoNews
fi
}

WriteEnv()
{
# Write new configuration values to .olmenurc file
echo "DoUpload=$DoUpload" > $HOME/.olmenurc
echo "DoDownload=$DoDownload" >> $HOME/.olmenurc
echo "DefaultProtocol=$DefaultProtocol" >> $HOME/.olmenurc
echo "ReplyFile=\"$ReplyFile\"" >> $HOME/.olmenurc
echo "NewFile=\"$NewFile\"" >> $HOME/.olmenurc
echo "TrashDir=\"$TrashDir\"" >> $HOME/.olmenurc
echo "DoMail=$DoMail" >> $HOME/.olmenurc
echo "DoNews=$DoNews" >> $HOME/.olmenurc
echo "Readonly=$Readonly" >> $HOME/.olmenurc
echo "UQ_MAX_BLKS=${UQ_MAX_BLKS}" >> $HOME/.olmenurc
}

KillFiles()
{
# List and optionally remove message-related files
# from the $TrashDir subdirectory
if [ -f $HOME/$TrashDir/* ]
        then Header "     Remove old files"
        echo "\n\t\tYou have the following files in $TrashDir:"
	echo
        ls $HOME/$TrashDir/*
        echo "\n\t\tWould you like to remove them? (y/n) \b"
        read reply
        case $reply in
        [Yy]*) rm $HOME/$TrashDir/*
               echo "\n\t\tYour files have been removed."
               ;;
            *) echo "\n\t\tYour files have not been removed"
               ;;
        esac
        sleep 1
fi
}

MoveFiles()
{
# Change security and move message-related files
# to $TrashDir directory
#
# Create an $TrashDir directory if it doesn't exist
if [ ! -d $HOME/$TrashDir ]
        then mkdir $HOME/$TrashDir
fi
# Make $NewFile read/write by owner only and move to $TrashDir
if [ -f $HOME/$NewFile ]
	then chmod 600 $HOME/$NewFile
        mv $NewFile $HOME/$TrashDir
fi
# Move message files if they exist
if [ -f AREAS ]
    then mv AREAS $HOME/$TrashDir
         mv *.MSG $HOME/$TrashDir
fi
# Move an old $ReplyFile if it exists
if [ -f $HOME/$ReplyFile ]
    then mv $HOME/$ReplyFile $HOME/$TrashDir
fi
# Move an old REPLIES file if it exists
if [ -f REPLIES ]
    then mv REPLIES *.MSG $HOME/$TrashDir
fi
}

Header()
{
# Syntax: Header "message"
# Clear the screen and write a header with "message" in the right
# corner above the dashline.
clear
echo $MainHeader
echo "$LOGNAME\t\t\t\t\t\t $1"
echo $Dashline
}

MainMenu()
{
# Main menu template
Header "     OLMenu Main Menu"
echo "\n\t\tSelect OLMenu Preferences"
echo "\n\t\tU) Upload Reply Packet:\t\t$DoUpload"
echo "\t\tD) Download New Packet:\t\t$DoDownload"
echo "\t\t   (Protocol is $DefaultProtocol)"
echo "\n\t\tM) Read Mail:\t\t\t$ShowMail"
echo "\t\tN) Read News:\t\t\t$ShowNews"
echo "\n\t\tR) UQWK in READONLY mode?\t$Readonly"
echo "\n\t\tC) Configure OLMenu"
echo "\n\t\tE) Execute OLMenu"
echo "\t\tQ) Quit\n"
}

ConfigMenu()
{
# Configuration menu template
Header " OLMenu Configuration"
echo "\n\t\tFilenames, Paths and Protocol"
echo "\n\t\tR) Reply Packet Name is:\t$ReplyFile"
echo "\t\tN) New Packet Name is:\t\t$NewFile"
echo "\t\tO) Old files moved to: \t\t$TrashDir"
echo "\t\tP) File transfer protocol is: \t$DefaultProtocol"
echo "\t\tS) Change New Packet Size: \t${UQ_MAX_BLKS} blocks"
echo "\n\t\tM) Main Menu\n"
}

DoMainMenu()
{
# Display main menu and get a selection
MainMenu
GetSelection
    case $reply in
    [Uu]*) if [ $DoUpload = YES ]
	   then DoUpload=NO
           else DoUpload=YES
	   fi			  
	   ;;
    [Dd]*) if [ $DoDownload = YES ]
           then DoDownload=NO
                ShowMail="---"
                ShowNews="---"
           else DoDownload=YES
                ShowMail=$DoMail
                ShowNews=$DoNews
           fi
           ;;
    [Mm]*) if [ $DoDownload = YES ]
           then if [ $DoMail = YES ]
                then DoMail=NO
		     ShowMail=NO
                     UQ_DO_MAIL=0
                else DoMail=YES
		     ShowMail=YES
                     UQ_DO_MAIL=1
                fi
           else ShowMail="---"
                UQ_DO_MAIL=0
           fi
           export UQ_DO_MAIL
           ;;
    [Nn]*) if [ $DoDownload = YES ]
           then if [ $DoNews = YES ]
                then DoNews=NO
		     ShowNews=NO
                     UQ_DO_NEWS=0
                else DoNews=YES
		     ShowNews=YES
                     UQ_DO_NEWS=1
                fi
           else ShowNews="---"
                UQ_DO_NEWS=0
           fi
	   export UQ_DO_NEWS
	   ;;
    [Rr]*) if [ $Readonly = YES ]
                then Readonly=NO
		UQ_READ_ONLY=0
                else Readonly=YES
		UQ_READ_ONLY=1
           fi
	   export UQ_READ_ONLY
           ;;
    [Cc]*) DoConfigMenu
           ;;
  [EeXx]*) KillFiles
           Upload
           RunUQWK
           Download
           MoveFiles
	   ;;
    [Qq]*) clear
	   WriteEnv
	   exit
	   ;;
    [Hh]*) clear; hytelnet ${OLMenuPath}/olmenu.howto
	   ;;
	*) BadSelection
	   ;;
    esac
}

DoConfigMenu()
{
# Get a selection from the Configuration Menu
while :
do
ConfigMenu
GetSelection
    case $reply in
    [Rr]*) GetRFname
           ;;
    [Nn]*) GetNFname
           ;;
    [Oo]*) GetTDname
           ;;
    [Pp]*) PSelect
           ;;
    [Ss]*) ChgBlkSize
           ;;
    [Mm]*) return
           ;;
    [Hh]*) clear; hytelnet ${OLMenuPath}/olmenu.howto
	   ;;
        *) return
           ;;
    esac
done
}

GetSelection()
{
# Input a selection from the menus
echo "\n\t\tEnter a selection, then press <enter>: "; read reply
}

BadSelection()
{
# Notify user of incorrect menu selection and continue
echo "\t\t$reply is not a valid selection"
sleep 1
}

GetRFname()
{
Header "    Reply Packet Name"
echo "\n\t\tReply Packet Name is $ReplyFile"
echo "\n\t\tEnter a new name and press <enter> or"
echo "\t\tpress <enter> for no change\n"
echo "\t\tEnter Reply Packet name: \b"
read NewName
case $NewName in
	[a-zA-Z0-9]*) ReplyFile=$NewName
esac
}

GetNFname()
{
Header "      New Packet Name"
echo "\n\t\tNew Packet Name is $NewFile"
echo "\n\t\tEnter a new name and press <enter> or"
echo "\t\tpress <enter> for no change\n" 
echo "\t\tEnter New Message Packet name: \b"
read NewName
case $NewName in
	[a-zA-Z0-9]*) NewFile=$NewName
esac
}

GetTDname()
{
Header "Change Directory Name"
echo "\n\t\tOld files are in the $TrashDir Directory"
echo "\n\t\tEnter a new name and press <enter> or"
echo "\t\tpress <enter> for no change\n" 
echo "\t\tEnter New Directory Name: \b"
read Newdir
case $Newdir in
[a-zA-Z0-9]*) echo "\n\t\t\t*WARNING*\n"
              echo "\t\tOLMenu will create $HOME/$Newdir"
              echo "\t\tand move all files from" 
	      echo "\t\t$HOME/$TrashDir to $HOME/$Newdir"
              echo "\t\tbefore removing $HOME/$TrashDir\n"
              echo "\t\tDo you want to do this? (Y/N) \b"
	      read reply
              if [ $reply = Y ] || [ $reply = y ]
                 then mkdir $HOME/$Newdir
                     mv $HOME/$TrashDir/* $HOME/$Newdir/
                     rmdir $HOME/$TrashDir
                     TrashDir=$Newdir
              fi
esac
}

PSelect()
{
# Display menu and select file transfer protocol
Header "      Select Protocol"
echo "\n\t\tThe current file transfer protocol is $DefaultProtocol"
echo "\n\t\tSelect a file transfer protocol, then press <enter>"
echo "\t\tor press <enter> for no change"
echo "\n\t\tX) Xmodem"
echo "\t\tY) Ymodem"
echo "\t\tZ) Zmodem"
echo "\t\tS) ftp server (automatic transfer)"
echo "\t\tF) ftp (manual transfer)"
echo
GetSelection
case $reply in
    [Xx]*)  DefaultProtocol=Xmodem
	    ;;
    [Yy]*)  DefaultProtocol=Ymodem
	    ;;
    [Zz]*)  DefaultProtocol=Zmodem
	    ;;
    [Ss]*)  DefaultProtocol=ftpserver
	    ;;
    [Ff]*)  DefaultProtocol=ftp
            ;;
    [Hh]*)  hytelnet ${OLMenuPath}/olmenu.howto
	    ;;
esac
}

ChgBlkSize()
{
Header "      New Packet Size"
echo "\n\t\tDefault New Packet size is 4000 blocks"
echo "\t\t(approximately 200k zipped)"
echo "\n\t\tCurrent New Packet size is ${UQ_MAX_BLKS} blocks"
echo "\n\t\tEnter a new size in blocks,"
echo "\t\t(0 for unlimited size) then <enter>"
echo "\t\tor <enter> for no change\n"
echo "\t\tSize = \b"
read NewBlockSize
case $NewBlockSize in
	[0-9]*) UQ_MAX_BLKS=$NewBlockSize
		export UQ_MAX_BLKS
		;;
esac
}

Upload()
{
# Upload a file using the selected protocol
# Modify the commands for the protocols if necessary
if [ $DoUpload = YES ]
then if [ -f $HOME/$ReplyFile ]
        then mv $HOME/$ReplyFile $HOME/$TrashDir
     fi
     if [ -f $HOME/REPLIES ]
        then mv $HOME/REPLIES $HOME/*.MSG $HOME/$TrashDir
     fi
Header "     Upload $ReplyFile"
echo "\nUpload $ReplyFile now...\n"
case $DefaultProtocol in
        Xmodem) xmodem -rb $HOME/$ReplyFile
		;;
        Ymodem) rb $HOME/$ReplyFile
		;;
        Zmodem) rz
		;;
     ftpserver) GetFTPserver
		;;
           ftp) GetFTP
                ;;
esac
fi
}

Download()
{
# Download a file using the selected protocol
# Modify the commands for the protocols if necessary
# The "read junk" statements handle extraneous characters that follow
# termination of a download
if [ $DoDownload = YES ]
    then zip $NewFile AREAS *.MSG
    sleep 1
    Header "     DOWNLOAD $NewFile"
    case $DefaultProtocol in
        Xmodem) xmodem -sb $HOME/$NewFile
		echo "Press Enter to Continue"
		read junk
		;;
        Ymodem) sb $HOME/$NewFile
                echo "Press Enter to Continue"
                read junk
		;;
        Zmodem) sz -w 8192 $HOME/$NewFile
		echo "Press Enter to Continue"
		read junk
		read junk
		read junk
		read junk
		;;
    fpt_server) PutFTPserver
                ;;
           ftp) PutFTP
		;;
esac
fi
}

SetUQWK()
{
# Set the environment variables that control UQWK
if [ $Readonly = NO ]
        then UQ_READ_ONLY=0
        else UQ_READ_ONLY=1
fi
if [ $DoDownload = NO ]
        then UQ_DO_MAIL=0
             UQ_DO_NEWS=0
        else if [ $DoMail = YES ]
                then UQ_DO_MAIL=1
                else UQ_DO_MAIL=0
             fi
             if [ $DoNews = YES ]
                then UQ_DO_NEWS=1
                else UQ_DO_NEWS=0
             fi
fi
export UQ_READ_ONLY UQ_DO_MAIL UQ_DO_NEWS
}

RunUQWK()
{
# Run the UQWK program using configuration from .offlinerc and .olmenurc
# and selections made in this program
if [ $DoUpload = YES ] || [ $DoDownload = YES ]
        then SetUQWK
        Header "         Execute UQWK"
        if [ $DoUpload = YES ]
        	then unzip -U $HOME/$ReplyFile
                     uqwk -RREPLIES
        	else uqwk
	fi
        sleep 1
fi
}

GetFTPserver()
{
# Get a file with telnet session and ftp server
# Requires ncftp - has not been especially reliable on our system
# Manual ftp transfers have been fine.
Header "ftp server transfer"
echo "\n\tFTP Server must be running for file transfer to occur,"
echo "\tand the file $ReplyFile must be in the public access directory."
echo "\n\tStart FTP Server in another window if necessary."
echo "\n\tPress enter in this window to transfer the file."
read keypress
ncftp `finger -s $LOGNAME| grep pm| nawk '{print $7}'`:$ReplyFile
}

PutFTPserver()
{
# Send a file with a telnet session and ftp server
# See above note about ncftp and reliability
Header "ftp server transfer"
echo "\n\tFTP Server must be running for file transfer to occur."
echo "\tStart FTP Server in another window if necessary then"
echo "\tenter command in this window to transfer the file."
echo "\tCommands:"
echo "\t\tput $NewFile    # sends message packet"
echo "\t\tquit           # ends ftp session, returns to main menu\n"
ncftp -V 0 `finger -s $LOGNAME| grep pm| nawk '{print $7}'`
}

GetFTP()
{
# Get a file with a telnet session and ftp without server
Header "  ftp manual transfer"
echo "\n\t\tOLMenu is ready to receive $ReplyFile"
echo "\n\t\tSwitch to your ftp session and"
echo "\t\ttransfer $ReplyFile now"
echo "\n\t\tWhen the transfer is complete,"
echo "\t\tswitch back to this session and"
echo "\t\tpress enter to continue."
read keypress
}

PutFTP()
{
# Send a file with a telnet session and ftp without server
Header "  ftp manual transfer"
echo "\n\t\tOLMenu has prepared $NewFile"
echo "\n\t\tSwitch to your ftp session and"
echo "\t\ttransfer $NewFile now"
echo "\n\t\tWhen the transfer is complete,"
echo "\t\tswitch back to this session and"
echo "\t\tpress enter to continue."
read keypress
}

# Begin main program

# Get the environment variables from .olmenurc
ReadEnv

# Main program loop
while :
do
DoMainMenu
done #end of main while/do
